xquery version "1.0";

declare namespace xh = "http://www.w3.org/1999/xhtml";
declare namespace exist = "http://exist.sourceforge.net/NS/exist";
import module namespace request = "http://exist-db.org/xquery/request";
import module namespace response = "http://exist-db.org/xquery/response";
import module namespace transform = "http://exist-db.org/xquery/transform";
import module namespace sm = "http://exist-db.org/xquery/securitymanager";
import module namespace util = "http://exist-db.org/xquery/util";
import module namespace xmldb = "http://exist-db.org/xquery/xmldb";

declare variable $exist:path external;

declare function local:ignore() as element(exist:igore) {
    <exist:ignore>
        <exist:cache-control cache="yes"/>
    </exist:ignore>
};

declare function local:render($page as element(page)) as element(xh:html) {
    transform:transform(
        $page,    
        doc("/db/xml-amsterdam/presentation.xslt"),
        ()
    )
};

declare function local:home-page() as element(page) {
    <page>
        <content>
        {
            doc("/db/xml-amsterdam/home.xml")
        }
        </content>
        <events>
        {
            for $conference in collection("/db/xml-amsterdam/data")/conference[xs:date(date) ge current-date()]
            order by xs:date($conference/date) ascending
            return
                <event>
                {
                    $conference/@ref,
                    $conference/name,
                    $conference/date,
                    <summary>{substring($conference/description, 1, 50)}</summary>
                }
                </event>
        }
        </events>
    </page>
};

declare function local:add-conference-page() as element(page) {
    <page>
        <content>
        {
            doc("/db/xml-amsterdam/conference-form.xhtml")
        }
        </content>
    </page>
};

declare function local:category-list() as element(category-list) {
    <category-list>
        <category term="XML"/>
        <category term="Java"/>
        <category term="JavaScript"/>
        <category term="C++"/>
    </category-list>
};

declare function local:submit-conference($conference-doc as document-node()) as xs:boolean {
    let $short-name := concat(substring(lower-case(replace($conference-doc/conference/name, " ", "")), 1, 6), replace($conference-doc/conference/date, "-", "")) return
    
        let $stored-as := xmldb:store("/db/xml-amsterdam/data", fn:concat($short-name, ".xml"), $conference-doc),
        $null := sm:chmod($stored-as, "rwur--r--"),
        $null := update value doc($stored-as)/conference/@ref with $short-name return
        
            true()
};


(: main - process URIs :)
if($exist:path = ("", "/"))then
    
    local:render(local:home-page())

else if($exist:path = "/add-conference")then
    
    local:render(local:add-conference-page())
        
else if($exist:path = "/submit-conference")then
    
    let $saved := local:submit-conference(request:get-data()) return
        if($saved) then
            local:render(local:home-page())
        else
        (
            response:set-status-code(500),
            <error>Could not store and update conference entry</error>
        )
else if($exist:path = "/category/")then
    local:category-list()
    
else if(fn:starts-with($exist:path, "/data/") and request:get-parameter("json", ()))then
    let $null := util:declare-option("exist:serialize", "method=json") return
        fn:doc(fn:concat("/db/xml-amsterdam", $exist:path))
else
    local:ignore()